@@ -49,8 +49,6 @@ from PyQt5.QtWidgets import (QAbstractItemView,
                              QScrollArea,
                              )
 
-from .. import ChanTime
-from ..trans import montage, filter_, remove_artf_evts, _select_channels
 from ..attr import Annotations, create_empty_annotations
 from ..attr.annotations import create_annotation
 from ..detect import DetectSpindle, DetectSlowWave, merge_close
@@ -63,13 +61,16 @@ lg = getLogger(__name__)
 
 MAX_FREQUENCY_OF_INTEREST = 50
 
-STAGE_SHORTCUT = ['1', '2', '3', '5', '9', '8', '0', '', '', '']
+STAGE_SHORTCUT = ['1', '2', '3', '5', '9', '8', '0', '', '', '7']
 QUALIFIERS = ['Good', 'Poor']
 QUALITY_SHORTCUT = ['o', 'p']
-SPINDLE_METHODS = ['Wamsley2012', 
+SPINDLE_METHODS = ['Moelle2011',
+                   'Ray2015',
+                   'Wamsley2012',  
                    'Nir2011', 
                    'Ferrarelli2007',
-                   'Moelle2002', 
+                   'FASST',
+                   'FASST2',
                    'UCSD', 
                    'Concordia',
                    ]
@@ -389,11 +390,13 @@ class Notes(QTabWidget):
 
         act = QAction('Spindle...', self)
         act.triggered.connect(self.parent.show_spindle_dialog)
+        act.setShortcut('Ctrl+Shift+s')
         act.setEnabled(False)
         actions['spindle'] = act
 
         act = QAction('Slow wave...', self)
         act.triggered.connect(self.parent.show_slow_wave_dialog)
+        act.setShortcut('Ctrl+Shift+w')
         act.setEnabled(False)
         actions['slow_wave'] = act
 
@@ -1214,6 +1217,8 @@ class Notes(QTabWidget):
 
             if method in ['Wamsley2012', 'UCSD']:
                 detector.det_wavelet['dur'] = params['win_sz']
+            elif method == 'Ray2015':
+                detector.zwin['dur'] = params['win_sz']
             else:
                 detector.moving_rms['dur'] = params['win_sz']
 
